<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class ObraHerramientaController extends AuthController {

  public $modelClass = "v1\models\ObraHerramienta";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));

    $query = $this->queryInicial;

    if($id > 0) {
      $query->andWhere(["id" => $id]);
    }

    if($buscar) {
      
      $query->andWhere([
        "OR",
        ["ilike", "nombre", $buscar]
      ]);

    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }
  public function actionGuardar()
  {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;
    $nombre = $this->req->getBodyParam("idHerramienta", "");

    if ($id !== "") {
      $modelo = $this->modelClass::findOne($id);
    }
    if ($modelo === null) {
      $modelo = new $this->modelClass();
      $modelo->uuid();
      $modelo->creado = new Expression('now()');
    } else {
      $modelo->modificado = new Expression('now()');
    }

   
    $modelo->load($this->req->getBodyParams(), '');
    
    $registroExistente = $this->modelClass::find()
    ->where(['idHerramienta' => $nombre])
    ->andWhere(['!=', 'id', $id]) 
    ->andWhere(['eliminado'=>null])// Excluir el registro actual en caso de edición
    ->one();    
    
    if ($registroExistente !== null) {
      return (new Respuesta($modelo))
         ->esError()
          ->mensaje("La Herramienta ya existe. Por favor, ingrese una Herramienta diferente.");
  }
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("Hubo un problema al guardar el Gasto");
    }

    $modelo->refresh();
    return (new Respuesta($modelo))
      ->mensaje("Gasto guardado");
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;

    if($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Herramienta no encontrada");
    }
    $modelo->eliminado = new Expression('now()');
    if(!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar la Herramienta");
    }

    return (new Respuesta())
      ->mensaje("Herramienta eliminada");
  }
}